package simulation;

import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

import commandSystem.Direction;
import commandSystem.ElevatorListener;


public class Elevator {
	static final int PRECISION = 10;
	private double floorHeight;
	private double step;
	private double height;
	// private int currentFloor = 0;
	// private boolean betweenFloors = false; /* Between currentFloor and the one above it. */
	private Direction direction = Direction.NONE;
	private boolean stoppingNextFloor = false;
	private boolean emergency = false;
	private ElevatorListener listener;

	public Elevator(int currentFloor, int nbFloors) {
		height = (double) currentFloor / nbFloors;
		floorHeight = 1. / nbFloors;
		step = floorHeight / PRECISION;
		System.out.println("step: " + step + ", floorHeight: " + floorHeight);
	}

	public void setActionListener(ElevatorListener listener) {
		this.listener = listener;
	}

	public void goUp() {
		if (emergency) return;
		stoppingNextFloor = false;
		direction = Direction.UP;
	}

	public void goDown() {
		if (emergency) return;
		stoppingNextFloor = false;
		direction = Direction.DOWN;
	}

	public void stop() {
		direction = Direction.NONE;
	}

	public void stopNextFloor() {
		stoppingNextFloor = true;
	}

	public void emergencyStop() {
		emergency = true;
		direction = Direction.NONE;
	}

	public void cancelEmergency() {
		emergency = false;
	}

	public void update() {
		if (emergency) return;
		int prevFloor = (int) Math.floor(height / floorHeight);
		switch (direction) {
		case DOWN:
			height -= step;

			/* Bottom reached. */
			if (height < step) {
				height = 0;
				direction = Direction.NONE;
			}

			/* Passed a floor, invoke the listener. */
			if (Math.floor(height / floorHeight) < prevFloor && listener != null) {
				listener.floorChange(Direction.DOWN);
			}
			// /* Go halfway below. */
			// if (!betweenFloors) currentFloor--;
			// betweenFloors = !betweenFloors;
			break;
		case UP:
			height += step;

			/* Top reached. */
			if (height > 1-step) {
				height = 1;
				direction = Direction.NONE;
			}

			/* Passed a floor, invoke the listener. */
			if (Math.floor(height / floorHeight) > prevFloor && listener != null) {
				listener.floorChange(Direction.UP);
			}
			// /* Go halfway above. */
			// if (betweenFloors) currentFloor++;
			// betweenFloors = !betweenFloors;
			break;
		default:
			break;
		}
	}

	// public int getCurrentFloor() {
	// 	return currentFloor;
	// }

	// public boolean getBetweenFloors() {
	// 	return betweenFloors;
	// }

	public Direction getDirection() {
		return direction;
	}

	public boolean getStoppingNextFloor() {
		return stoppingNextFloor;
	}

	public double getHeight() {
		return height;
	}

	public boolean getEmergency() {
		return emergency;
	}
}
